from pydantic import BaseModel
from restack_ai.function import function
from openai import OpenAI
from ..utils.toolhouse_client import toolhouse_client

class MailInput(BaseModel):
    subject: str
    body: str
    to: str
    openai_api_key: str | None = None
    toolhouse_api_key: str | None = None
    model: str | None = None


@function.defn(name="mail_website_summary")
async def mail_website_summary(input: MailInput) -> str:
    client = OpenAI(api_key=input.openai_api_key)
    th = toolhouse_client(api_key=input.toolhouse_api_key, provider="openai")

    messages = [
        {"role": "user", "content": f"Send email to {input.to} with the following subject: {input.subject} and body: {input.body}"},
    ]  

    chat_params = {
        "model": input.model or "gpt-4o-mini",
        "messages": messages,
    } 

    response = client.chat.completions.create(
       **chat_params,
        tools=th.get_tools()
    )

    messages += th.run_tools(response, append=True)

    print(messages)

    client.chat.completions.create(
        model="gpt-4o",
        messages=messages,
        tools=th.get_tools()
    )

    return "Email sent successfully"



from toolhouse import Toolhouse
import os

def toolhouse_client(api_key: str | None = os.getenv("TOOLHOUSE_API_KEY"), provider: str = "openai") -> Toolhouse:
    if not api_key:
        raise ValueError("API key is required to create Toolhouse client")
    return Toolhouse(provider=provider, api_key=api_key)



from restack_ai import Restack
from pydantic import BaseModel
from .functions.send_email import mail_website_summary
from .functions.summarize_website import summarize_website
from .task_queue import toolhouse_task_queue
class ToolhouseServiceOptions(BaseModel):
    rate_limit: int

class RestackWrapper(BaseModel):
    client: Restack
    
    class Config:
        arbitrary_types_allowed = True


class ToolhouseServiceInput(BaseModel):
    client: RestackWrapper
    options: ToolhouseServiceOptions


async def toolhouse_service(input: ToolhouseServiceInput):
    return await input.client.start_service(
        functions=[
            mail_website_summary,
            summarize_website
        ],
         task_queue=toolhouse_task_queue,
        options=input.options
    )

if __name__ == "__main__":
    toolhouse_service(
        client=Restack(),
        options=ToolhouseServiceOptions(rate_limit=100000)
    )
